SystemOrganization addCategory: #'Installer-Launcher'!

!SystemNavigation class methodsFor: '*installer-launcher' stamp: 'kph 2/27/2007 19:00'!
launchFrom: launcher

^launcher reportFor: self default! !

!SystemNavigation class methodsFor: '*installer-launcher' stamp: 'kph 2/16/2007 10:01'!
launchHelp

^'report=method            Execute the query method and print the results to stdout or file.
to="*.txt"               Filename or pattern to write report to (default is stdout).
'! !

Object subclass: #Launcher
	instanceVariableNames: 'nextParameterIndex image mock commandLineClass actionSelector nextCommandLineDocument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Installer-Launcher'!

!Launcher commentStamp: 'testReporter 3/1/2007 19:06' prior: 0!
A Launcher provides squeak with a range of capabilities that are intended to be used from the command line.

As a design goal, the startup process can invoke Launcher several times at several points in the startup sequence. In particular 1) below aims to provide a mechanism for getting in to images that are broken in some form, so as to enable a script to be run before morphic has attempted to startup.

1) Launcher is registered to handle startup notification after Security manager
any class which wishes to provide a service to be usable at this time implements startFrom:

2) Launcher is wired in to ProjectLauncher to handle startup notification when squeak is fully initialized. Any class wishing to provide a service to be usable at this time implements launchFrom:

#startFrom: and #launchFrom: are called providing a handle to the launcher which may be used to obtain parameters using the desired scheme. 

a) launcher getParameters . key=value +yes -no
b) launcher getParametersOldStyle . key value key2 value2

Instance Variables
	actionSelector: #startFrom: #launchFrom:
	commandLineClass: aClass
	image:	the smalltalk image or self if being a mock
	mock: the mock array of startup parameters
	nextCommandLineDocument: aString
	nextParameterIndex:	 anInteger
 !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'testReporter 3/1/2007 16:09'!
initialize
 
	Smalltalk addToStartUpList: self after: SecurityManager.
! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'testReporter 3/1/2007 16:20'!
launchFrom: launcher

	^self launchWith: launcher getParameters			! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'testReporter 3/1/2007 16:20'!
launchHelp

"
Launcher testLaunch: #('+help').
"

^'+help,h                  Prints this help text.
scripts="<sc.st> <sc2.st>"   Execute smalltalk scripts given by urls.'
! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'klc 8/14/2008 14:17'!
launchWith: params

	params at: 'H' ifPresent: [:v | params at: 'HELP' put: v ].
	params at: 'S' ifPresent: [ :v | params at: 'SCRIPTS' put: v ].
	params at: 'HELP' ifPresent: [ :v | Script new writeHelp ].
	params at: 'SCRIPTS' ifPresent: [:v |
		CodeLoader new loadSourceFiles: (Array withAll: (v findTokens: ' ')) ;
			installSourceFiles].
	^true! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'testReporter 3/1/2007 19:09'!
newLaunchWith: arrayOrImage
 
	^ (self new) 
		actionSelector: #launchFrom: ;
		image: arrayOrImage;
		commandLineClass: self;
		begin;
		yourself
	

	
	! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'testReporter 3/1/2007 19:09'!
newStartWith: arrayOrImage
 
	^ (self new) 
		actionSelector: #startFrom: ;
		image: arrayOrImage;
		commandLineClass: self;
		begin;
		yourself
	

	! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/28/2007 01:05'!
startFrom: starter

	^self startWith: starter getParameters! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:04'!
startUp: resuming
"we are sent this when starting as an image startup item before autostart"

	^ self newStartWith: SmalltalkImage current ! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 9/1/2008 20:22'!
startWith: params

	params at: 'START' ifPresent: [:v | 
		CodeLoader new loadSourceFiles: (Array withAll: (v findTokens: ' ')) ;
			installSourceFiles ].
	^true! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:05'!
actionSelector: aSelector

	actionSelector := aSelector! !

!Launcher methodsFor: 'public' stamp: 'kph 2/27/2007 18:13'!
beFirstTime 

nextParameterIndex := 3.
 ! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 3/2/2007 16:50'!
begin 

	nextCommandLineDocument := nil.
 	
	[ self launch: commandLineClass ] 
		whileTrue: [ 
			commandLineClass := self nextCommandLineClass.
			commandLineClass ifNil: [ ^self ] ].
	
	  
			
			
			
			! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:05'!
classes

	^ Smalltalk! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:05'!
commandLineClass: aClass

	commandLineClass := aClass.
 			
			! !

!Launcher methodsFor: 'public' stamp: 'testReporter 3/1/2007 17:28'!
getParameters 

	"this utility extracts the unprocessed parameters into a dictionary from the typical form:
	
	param1=true param2=''hello world'' +param3 -param4

	i.e. items with an = are parameters,
	+ gives param3=true
	- give param4=false

	items without +=- are considered to refer to the next Launch Class or document.
	"

	| key value parameters item |
	
	parameters := Dictionary new.
	
	parameters at: #document put: commandLineClass.
 
	[ (item := image getSystemAttribute: nextParameterIndex) isEmptyOrNil] 
		whileFalse:[
			nextParameterIndex := nextParameterIndex + 1.
			item := item readStream.
			value := nil.
			(item peek = $-) ifTrue: [ value := false. item next  ].
			(item peek = $+) ifTrue: [ value := true.  item next  ].
			key := item upTo: $=.
			value isNil & item atEnd not ifTrue: [ value := item upToEnd ].
			value 
				ifNotNil: [ parameters at: key asUppercase put: value ] 
				ifNil: [ nextCommandLineDocument  := key. 
					    ^parameters ].
	].
	
	nextCommandLineDocument := nil.
	^parameters

			
			
			
			! !

!Launcher methodsFor: 'public' stamp: 'testReporter 3/1/2007 17:28'!
getParametersOldStyle

	"this utility extracts the unprocessed parameters into a dictionary from the typical old style form:
	
	KEY value Key2 Value Key3 Value :NextLauncherClass KEY value Key2 value etc.
	
	the ability to specify the next launcher class by beginning a parameter 
	with a non letter character has been added.
	"

	| key value parameters |
	
	parameters := Dictionary new.
	
	parameters at: #document put: commandLineClass.
 
	[ (key := image getSystemAttribute: nextParameterIndex) isEmptyOrNil] 
		whileFalse:[
			nextParameterIndex := nextParameterIndex + 1.
			key first isLetter ifFalse: [ 
				nextCommandLineDocument :=  key copyWithoutFirst. 
				^ parameters ].
			
			value := image getSystemAttribute: nextParameterIndex.
			nextParameterIndex := nextParameterIndex + 1.
			
			parameters at: key asUppercase put: value. 
	].

	^parameters

			
			
			
			! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:05'!
getSystemAttribute: n
"this provides the ability for us to be a mock image with startup parameters in our array."

	^ mock at: n - 1 ifAbsent: nil! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:06'!
image: imageOrMock

	image := imageOrMock isCollection  
		ifTrue: [ mock := imageOrMock. self ]
		ifFalse: [ imageOrMock ] .

 ! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:06'!
initialize

	nextParameterIndex := 2! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:06'!
launch: aClass

	^ (aClass respondsTo: actionSelector) 
		ifTrue: [ aClass perform: actionSelector with: self ]
		ifFalse: [ false ]! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/27/2007 18:47'!
nextCommandLineClass
			
	^nextCommandLineDocument ifNotNilDo: [:l | self classes at: l asSymbol ifAbsent: [ nil ] ]
	
 ! !

!Launcher methodsFor: 'reporting' stamp: 'mtf 7/3/2008 15:08'!
report: reportSelector for: item on: stream

	(item respondsTo: reportSelector) ifFalse: [ ^ stream nextPutAll: 'REPORT UNAVAILABLE'; cr ].
	(reportSelector last = $:) ifTrue: [ ^item perform: reportSelector with: stream ].
	(item perform: reportSelector) ifNotNilDo: [ :result | 
		^result isString
			ifTrue: [ stream nextPutAll: result ] 
			ifFalse: [ result do: [ :each | stream nextPutAll: each asString; cr ] ] ]! !

!Launcher methodsFor: 'reporting' stamp: 'mtf 7/3/2008 15:08'!
reportFor: target  

	^ self reportFor: target with: self getParameters! !

!Launcher methodsFor: 'reporting' stamp: 'mtf 7/3/2008 15:10'!
reportFor: target with: params
 
	| reports dest scriptIO quit |
	reports := params at: 'REPORT' ifAbsent: [ ^ self error: 'no report requested' ].
	reports := reports findTokens: ' '.
	dest := params at: 'TO' ifAbsent: nil.
	scriptIO := Script new writeTo: (params at: 'TO' ifAbsent: nil) in: (params at: 'DIR' ifAbsent: '').

	reports do: [ : reportSelector |
		scriptIO := Script new 
			writeTo: (dest ifNotNil: [ dest copyReplaceAll:'*' with: reportSelector]) 
			in: (params at: 'DIR' ifAbsent: '').
		scriptIO printWith: [ :out | self report: reportSelector asSymbol for: target on: out ] ].
 
	quit := params at: 'QUIT' ifAbsent: [ false ].
	quit ~= false ifTrue: [ SmalltalkImage current snapshot: false andQuit: true ].
	^true! !

Object subclass: #Script
	instanceVariableNames: 'stderr stdin stdout fileout dir'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Installer-Launcher'!

!Script class methodsFor: 'as yet unclassified' stamp: 'kph 2/27/2007 18:57'!
launchFrom: launcher

	^self launchWith: launcher getParameters! !

!Script class methodsFor: 'as yet unclassified' stamp: 'kph 2/22/2007 16:00'!
launchHelp

^
'eval, e="2+2"            Evaluate code and quit.
print, p="2+2"           Evaluate and print result to stdout and quit.
-quit                    Remain open after evaluating code'! !

!Script class methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:01'!
launchWith: params

	| script quit |
	params at: 'E' ifPresent: [:v | params at: 'EVAL' put: v ].
	params at: 'P' ifPresent: [:v | params at: 'PRINT' put: v ].

	script := self new.

	params at: 'TO' ifPresent: [ :v | script writeTo: v in: (params at: 'DIR' ifAbsent: '')].
	params at: 'HELP' ifPresent: [ :v | script writeHelp ].
	params at: 'EVAL' ifPresent: [ :v | script evaluate: v ].
	params at: 'PRINT' ifPresent: [ :v | script evaluateAndPrint: v ].

	quit := params at: 'QUIT' ifAbsent: true.
	quit ~= false ifTrue: [ SmalltalkImage current snapshot: false andQuit: true ].
	^true! !

!Script class methodsFor: 'as yet unclassified' stamp: 'testReporter 3/1/2007 16:23'!
startFrom: launcher

	^self startWith: launcher getParameters! !

!Script class methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:03'!
startWith: params

	| script |
	script := self new.
	params at: 'TO' ifPresent: [ :v | script writeTo: v in: (params at: 'DIR' ifAbsent: '')].
	params at: 'INIT' ifPresent: [ :v | script evaluate: v ].
	params at: 'INITP' ifPresent: [ :v | script evaluateAndPrint: v ].
	^true! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 04:09'!
evaluate: code

	^Compiler new evaluate: code in: nil to: self.! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 14:49'!
evaluateAndPrint: code

	self printWith: [ :output | output nextPutAll: (self evaluate: code) asString; cr ]
	 ! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 14:50'!
initialize

	stdout := (MultiByteFileStream new) 
		wantsLineEndConversion: #crlf; 
		open: '/dev/stdout' forWrite: true.
		
	stderr :=  (MultiByteFileStream new) 
		wantsLineEndConversion: #crlf; 
		open: '/dev/stderr' forWrite: true. 
	 
	stdin  := AsyncFile new open: '/dev/stdin' forWrite: false.! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 14:50'!
printWith: aBlock

	fileout ifNil: [ ^aBlock value: stdout ].
	FileStream detectFile: [ dir forceNewFileNamed: fileout ] do: aBlock ! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 14:50'!
stderr

	^stderr! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 14:51'!
stdout

	^stdout! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 15:10'!
writeHelp

	| helps |
	helps := SystemNavigation default allImplementorsOf: #launchHelp.
	self printWith: [ :output |	
		helps do: [ :mr | 
			output nextPutAll: mr classSymbol; cr; cr.
			output nextPutAll: (mr actualClass soleInstance launchHelp); cr; cr ] ]! !

!Script methodsFor: 'as yet unclassified' stamp: 'mtf 7/3/2008 14:56'!
writeTo: fileName in: directory

	fileout := fileName.
	dir := FileDirectory on: (FileDirectory default fullNameFor: directory).
	dir assureExistence.! !

!ProjectLauncher methodsFor: '*installer-launcher' stamp: 'kph 6/6/2008 23:42'!
startUpAfterLogin
	| scriptName loader isUrl |
	self setupFlaps.
	Preferences readDocumentAtStartup ifTrue: [
		HTTPClient isRunningInBrowser ifTrue:[
			self setupFromParameters.
			scriptName := self parameterAt: 'src'.
			CodeLoader defaultBaseURL: (self parameterAt: 'Base').
		] ifFalse:[
			scriptName := (SmalltalkImage current getSystemAttribute: 2) ifNil:[''].
			scriptName := [ scriptName convertFromSystemString ] ifError: [ scriptName asString ].
			scriptName isEmpty ifFalse:[
				"figure out if script name is a URL by itself"
				isUrl := (scriptName asLowercase beginsWith:'http://') or:[
						(scriptName asLowercase beginsWith:'file://') or:[
						(scriptName asLowercase beginsWith:'ftp://')]].
				isUrl ifFalse:[
					Smalltalk at: scriptName asSymbol
					ifPresent: [:launchClass | ^ Launcher newLaunchWith: SmalltalkImage current ].
					
					scriptName := 'file:', (FileDirectory default fullNameFor: scriptName) ]
				].
		]. ]
	ifFalse: [ scriptName := '' ].

	scriptName isEmptyOrNil
		ifTrue:[^Preferences eToyFriendly ifTrue: [self currentWorld addGlobalFlaps]].
	loader := CodeLoader new.
	loader loadSourceFiles: (Array with: scriptName).
	(scriptName asLowercase endsWith: '.pr') 
		ifTrue:[self installProjectFrom: loader]
		ifFalse:[loader installSourceFiles].
! !

!SmalltalkImage class methodsFor: '*installer-launcher' stamp: 'kph 2/27/2007 18:57'!
launchFrom: launcher

 ^self launchWith: launcher getParameters! !

!SmalltalkImage class methodsFor: '*installer-launcher' stamp: 'kph 2/16/2007 10:05'!
launchHelp

^
'+save                    Save image
save=name.image          Save image with given file name
+quit                    Quit squeak
'! !

!SmalltalkImage class methodsFor: '*installer-launcher' stamp: 'kph 1/18/2008 07:51'!
launchWith: params

	| saveAs quit resuming |

	saveAs := params at: 'SAVE' ifAbsent: false.
	quit := params at: 'QUIT' ifAbsent: false.

	(saveAs isKindOf: Boolean) 
		ifFalse: [
				saveAs = 'AUTO' ifTrue: [ saveAs := SystemVersion current version ,'-', Date today yyyymmdd ].
			
				  (saveAs endsWith: FileDirectory imageSuffix) ifFalse: [ saveAs := saveAs, '.', FileDirectory imageSuffix ]. 
				  resuming := self current saveAs: saveAs.
				  resuming ifTrue: [ ^false ].
				  saveAs := false ].
		
 	resuming := self current snapshot: saveAs andQuit: (quit == true).
	 
	resuming ifTrue: [ ^false ].
	"returning false, stops evaluation of further commandline parameters
	 this will be beneficial for image restart"

	^ true
	! !
Launcher initialize!
